package SPVM::Time::Info;

1;

=head1 Name

SPVM::Time::Info - struct tm in C language (Removed)

=head1 Description

This class is removed. Use the L<Sys::Time::Tm|SPVM::Sys::Time::Tm> class instead.
